/**
 * @constructor Apemandocker
 * @param {object} config
 */
"use strict";

const execcli = require('execcli'),
    async = require('async'),
    os = require('os'),
    childProcess = require('child_process'),
    argx = require('argx'),
    stringcase = require('stringcase'),
    extend = require('extend');

/** @lends Apemandocker */
function Apemandocker(config) {
    let s = this;
    extend(s, config || {});
}

Apemandocker.prototype = {
    _docker: function (args, callback) {
        execcli('docker', args, {
            notfound: `install docker from https://docs.docker.com`
        }, callback);
    },
    /**
     * Build docker source.
     * @param {string} source - Name of source.
     * @param {object} [options] - Optional settings.
     * @param {function} callback - Callback when done.
     */
    buildImage: function (source, options, callback) {
        let s = this;
        let args = argx(arguments);
        callback = args.pop('function') || argx.noop;
        options = args.pop('object') || {};
        s._docker(['build', options, source], callback);
    },
    /**
     * Remove docker image.
     * @param {string} image - Name of image.
     * @param {object} [options] - Optional settings.
     * @param {function} callback - Callback when done.
     */
    removeImage: function (image, options, callback) {
        let s = this;
        let args = argx(arguments);
        callback = args.pop('function') || argx.noop;
        options = args.pop('object') || {};
        s._docker(['rmi', options, image], callback);
    },
    listImages: function (options, callback) {
        let s = this;
        let args = argx(arguments);
        callback = args.pop('function') || argx.noop;
        options = args.pop('object') || {};

        async.waterfall([
            (callback) => {
                let command = `docker images`;
                childProcess.exec(command, callback);
            },
            (stdOut, stdErr, callback) => {
                if (stdErr) {
                    console.error(stdErr);
                }
                let lines = String(stdOut).trim().split(os.EOL);

                let headers = lines.shift().split(/\s\s+/g).map((key) => {
                    return stringcase.camelcase(String(key).trim().replace(/\s/g, '_'));
                });

                let results = lines.map((line) => {
                    let data = {};
                    line.split(/\s\s+/g).forEach((val, i) => {
                        let key = headers[i];
                        data[key] = val;
                    });
                    return data;
                });
                callback(null);
            }
        ], callback);
    },
    /**
     * Create a container.
     * @param {string} image - Name of image.
     * @param {object} [options] - Optional settings.
     * @param {function} callback - Callback when done.
     */
    createContainer: function (image, options, callback) {
        let s = this;
        let args = argx(arguments);
        callback = args.pop('function') || argx.noop;
        options = args.pop('object') || {};
        s._docker(['create', options, image], callback);
    },
    /**
     * Remove a container.
     * @param {string} container - Name of container.
     * @param {object} [options] - Optional settings.
     * @param {function} callback - Callback when done.
     */
    removeContainer: function (container, options, callback) {
        let s = this;
        let args = argx(arguments);
        callback = args.pop('function') || argx.noop;
        options = args.pop('object') || {};
        s._docker(['rm', options, container], callback);
    },
    /**
     * Start a container.
     * @param {string} container - Name of container.
     * @param {object} [options] - Optional settings.
     * @param {function} callback - Callback when done.
     */
    startContainer: function (container, options, callback) {
        let s = this;
        let args = argx(arguments);
        callback = args.pop('function') || argx.noop;
        options = args.pop('object') || {};
        s._docker(['start', options, container], callback);
    },
    /**
     * Stop a container.
     * @param {string} container - Name of container.
     * @param {object} [options] - Optional settings.
     * @param {function} callback - Callback when done.
     */
    stopContainer: function (container, options, callback) {
        let s = this;
        let args = argx(arguments);
        callback = args.pop('function') || argx.noop;
        options = args.pop('object') || {};
        s._docker(['stop', options, container], callback);
    },
    listContainers: function (options, callback) {
        let s = this;
        let args = argx(arguments);
        callback = args.pop('function') || argx.noop;
        options = args.pop('object') || {};

        let all = !!options.all;

        const keys = [
            "ID",
            "Image",
            "Command",
            "CreatedAt",
            "RunningFor",
            "Ports",
            "Status",
            "Size",
            "Names",
            "Labels"
        ];
        let joiner = ', ';
        async.waterfall([
            (callback) => {
                let format = keys.map((key) => {
                    return `{{.${key}}}`;
                }).join(joiner);
                let command = `docker ps${all ? ' -a' : ''} --format "${format}"`;
                childProcess.exec(command, callback);
            },
            (stdOut, stdErr, callback) => {
                if (stdErr) {
                    console.error(stdErr);
                }
                let results = String(stdOut).trim().split(os.EOL)
                    .filter((line) => {
                        return !!String(line).trim();
                    })
                    .map((line) => {
                        let data = {};
                        String(line).split(joiner).forEach((value, i) => {
                            let key = stringcase.camelcase(String(keys[i]).trim().replace(/\s/g, '_'))
                            data[key] = String(value).trim();
                        });
                        return data;
                    });
                callback(null, results);
            }
        ], callback);

    }
};

module.exports = Apemandocker;
